/*                                                           12-23-95 04:43pm
 ****************************************************************************
 * AVL NetWare C Interface Library     Version 1.02.002      01-02-96 02:54pm
 ****************************************************************************
 *
 *   BINDFUNC.C       Bindary Access Functions
 *
 *   Prototypes:
 *            int NetWare_init_scan_object
 *                  ( void );
 *            int NetWare_init_scan_property
 *                  ( void );
 *            int NetWare_init_scan_value
 *                  ( void );
 *            int NetWare_get_next_object
 *                  ( netware_objrequest_t  *object_request,
 *                    netware_objreply_t    *object_reply );
 *            int NetWare_get_next_property
 *                  ( netware_proprequest_t *property_request,
 *                    netware_propreply_t   *property_reply,
 *                    netware_objreply_t    *object_reply );
 *            int NetWare_get_value
 *                  ( netware_valrequest_t  *value_request,
 *                    netware_valreply_t    *value_reply,
 *                    netware_propreply_t   *property_reply,
 *                    netware_objreply_t    *object_reply );
 *
 ****************************************************************************
 *
 * Copyright (c) 1995,1996 Alexander V. Loukyanets
 * All rights reserved.
 *
 ****************************************************************************
 *
 * Compiler :  Zortech C/C++ 3.1r2 (Patched)
 *
 ****************************************************************************
 */

#include    <string.h>
#if defined( DEBUGGING )
#include    <stdio.h>
#endif
#include    <dos.h>
#include    "netware.h"


#pragma ZTC align 1


static char    _netware_all_ffs[48] =
{ 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
  0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
  0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
  0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
  0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
  0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
  0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
  0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF };
static int     _netware_obj_1st_time  = 1;
static int     _netware_prop_1st_time = 1;
static int     _netware_val_1st_time  = 1;


/*
 ****************************************************************************
 */

int NetWare_init_scan_object( void )
{
#if defined( DEBUGGING )
  printf( "\nNetWare_TRACE : NetWare_init_scan_object()" );
#endif
  _netware_obj_1st_time = 1;
}

/*
 ****************************************************************************
 */

int NetWare_init_scan_property( void )
{
#if defined( DEBUGGING )
  printf( "\nNetWare_TRACE : NetWare_init_scan_property()" );
#endif
  _netware_prop_1st_time = 1;
}

/*
 ****************************************************************************
 */

int NetWare_init_scan_value( void )
{
#if defined( DEBUGGING )
  printf( "\nNetWare_TRACE : NetWare_init_scan_value()" );
#endif
  _netware_val_1st_time = 1;
}

/*
 ****************************************************************************
 */

int NetWare_get_next_object( netware_objrequest_t *object_request,
                             netware_objreply_t   *object_reply )
{
  struct SREGS    segregs;
  union  REGS     regs;

#if defined( DEBUGGING )
  printf( "\nNetWare_TRACE : NetWare_get_next_object( %p, %p )", object_request, object_reply );
#endif

  if ( _netware_obj_1st_time ) {
       _netware_obj_1st_time = 0;
    memcpy( &object_reply->object_id, &_netware_all_ffs, 4 );
  }

  object_reply->buffer_length   = sizeof( netware_objreply_t ) - 2;

  object_request->buffer_length = sizeof( netware_objrequest_t ) - 2;
  object_request->request_type  = 0x37;
  memcpy( &object_request->last_object_id, &object_reply->object_id, 4 );
  memcpy( &object_request->object_type, &_netware_all_ffs, 2 );
  strcpy(  object_request->object_name, "*" );
  object_request->name_length = 48;

  regs.h.ah   = 0xE3;
  segregs.ds  = FP_SEG( (void _far *)object_request );
  regs.x.si   = FP_OFF( (void _far *)object_request );
  segregs.es  = FP_SEG( (void _far *)object_reply );
  regs.x.di   = FP_OFF( (void _far *)object_reply );
  int86x(0x21, &regs, &regs, &segregs);

  regs.h.ah = 0;
  return( regs.x.ax );
}

/*
 ****************************************************************************
 */

int NetWare_get_next_property( netware_proprequest_t *property_request,
                               netware_propreply_t   *property_reply,
                               netware_objreply_t    *object_reply )
{
  struct SREGS    segregs;
  union  REGS     regs;

#if defined( DEBUGGING )
  printf( "\nNetWare_TRACE : NetWare_get_next_property( %p, %p, %p )", property_request, property_reply, object_reply );
#endif

  if ( _netware_prop_1st_time ) {
       _netware_prop_1st_time = 0;
    memcpy( &property_reply->sequence_number, &_netware_all_ffs, 4 );
  }

  property_request->buffer_length = sizeof( netware_proprequest_t ) - 2;
  property_request->request_type  = 0x3C;
  memcpy( &property_request->object_type, &object_reply->object_type, 2 );
  property_request->object_name_length = 48;
  memcpy( &property_request->object_name, &object_reply->object_name, 48 );
  memcpy( &property_request->sequence_number,
          &property_reply->sequence_number, 4 );
  property_request->property_name_length = 16;
  strcpy(  property_request->property_name, "*" );

  property_reply->buffer_length   = sizeof( netware_propreply_t ) - 2;

  regs.h.ah   = 0xE3;
  segregs.ds  = FP_SEG( (void _far *)property_request );
  regs.x.si   = FP_OFF( (void _far *)property_request );
  segregs.es  = FP_SEG( (void _far *)property_reply );
  regs.x.di   = FP_OFF( (void _far *)property_reply );
  int86x( 0x21, &regs, &regs, &segregs );

  regs.h.ah = 0;
  return( regs.x.ax );
}

/*
 ****************************************************************************
 */

int NetWare_get_value( netware_valrequest_t *value_request,
                       netware_valreply_t   *value_reply,
                       netware_propreply_t  *property_reply,
                       netware_objreply_t   *object_reply )
{
  struct SREGS      segregs;
  union  REGS       regs;

#if defined( DEBUGGING )
  printf( "\nNetWare_TRACE : NetWare_get_value( %p, %p, %p, %p )", value_request, value_reply, property_reply, object_reply );
#endif

  if ( _netware_val_1st_time ) {
       _netware_val_1st_time = 0;
    value_request->segment_number = 0;
  }

  value_request->buffer_length = sizeof( netware_valrequest_t ) - 2;
  value_request->request_type  = 0x3D;
  memcpy( &value_request->object_type, &object_reply->object_type, 2 );
  value_request->object_name_length = 48;
  strcpy( value_request->object_name, object_reply->object_name );
  value_request->segment_number++;
  value_request->property_name_length = 16;
  strcpy( value_request->property_name, property_reply->property_name );

  value_reply->buffer_length = sizeof( netware_valreply_t ) - 2;
  regs.h.ah   = 0xE3;
  segregs.ds  = FP_SEG( (void _far *)value_request );
  regs.x.si   = FP_OFF( (void _far *)value_request );
  segregs.es  = FP_SEG( (void _far *)value_reply );
  regs.x.di   = FP_OFF( (void _far *)value_reply );
  int86x(0x21, &regs, &regs, &segregs);

  regs.h.ah = 0;
  return( regs.x.ax );
}



#pragma ZTC align

/****************************************************************************
*** end of BINDFUNC.C ***/
